package editor;


import java.util.HashMap;

import javax.swing.Action;
import javax.swing.SizeRequirements;
import javax.swing.text.Element;
import javax.swing.text.View;
import javax.swing.text.ViewFactory;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.InlineView;
import javax.swing.text.html.ParagraphView;

/**
 * Custom HTMLEditorKit class implementing letter-based wrap as described
 * at http://java-sl.com/tip_html_letter_wrap.html
 * 
 * @author Stanislav Lapitsky 
 */
@SuppressWarnings("serial")
public class ExtendedHTMLKit extends HTMLEditorKit {
	private HashMap<Object, Action> actions;    //map of name of ready-made action to action itself
	
	/**
	 * Default constructor. Builds the command-action map.
	 */
	public ExtendedHTMLKit () {
		//Construct the kit
		super();
		
		//Variables
		final Action[] kitActions;		//ready-made actions supported by current editor Kit
		Action kitAction;				//one of the ready-made actions
		int i;
		
		//Retrieves the list of commands (ready-made actions).
	    actions = new HashMap<Object, Action>();
	    kitActions = getActions();
	    for (i = 0; i < kitActions.length; i++){
	        kitAction = kitActions[i];
	        actions.put (kitAction.getValue(Action.NAME), kitAction);
	    }
	}
	
	/**
	 * Execute a given kit command corresponding to a certain ready-made action
	 * @param sCommand Name of the command (ready-made action)
	 */
	public void runCommand (String sCommand) {
		actions.get(sCommand).actionPerformed (null);
	}
	
	@Override 
    public ViewFactory getViewFactory() { 
        return new HTMLFactory() { 
            public View create(Element e) { 
               View v = super.create(e); 
               if (v instanceof InlineView) { 
                  return new InlineView(e) { 
                       public int getBreakWeight(int axis, float pos, float len) { 
                           return GoodBreakWeight; 
                       } 
                       public View breakView(int axis, int p0, float pos, float len) { 
                           if (axis == View.X_AXIS) { 
                              checkPainter(); 
                              int p1 = getGlyphPainter().getBoundedPosition (this, p0, pos, len); 
                              if (p0 == getStartOffset() && p1 == getEndOffset()) { 
                                 return this; 
                              } 
                              return createFragment(p0, p1); 
                           } 
                           return this; 
                       } 
                  }; 
               } else if (v instanceof ParagraphView) { 
                         return new ParagraphView(e) { 
                              protected SizeRequirements calculateMinorAxisRequirements(int axis, SizeRequirements r) { 
                                  if (r == null) { 
                                     r = new SizeRequirements(); 
                                  } 
                                  float pref = layoutPool.getPreferredSpan(axis); 
                                  float min = layoutPool.getMinimumSpan(axis); 
                           
                                  // Don't include insets, Box.getXXXSpan will include them. 
                                  r.minimum = (int)min; 
                                  r.preferred = Math.max(r.minimum, (int) pref); 
                                  r.maximum = Integer.MAX_VALUE; 
                                  r.alignment = 0.5f; 
                                  return r; 
                         } 

                     }; 
               } 
               return v; 
             } 
         }; 
    }
}
